#include "stdafx.h"
#include "EServ.h"
#include "AddAlias.h"
#include "AccessRights.h"

inline void convert (LPTSTR str, TCHAR from, TCHAR to) throw ()
{
	for (TCHAR* p = str; *p != _T('\0'); p++)
		if (*p == from)
			*p = to;
}

inline bool is_at_end (LPCTSTR what, LPCTSTR end) throw ()
{
	int delta = lstrlen (what) - lstrlen (end);
	return ((delta >= 0) && !lstrcmpi (what + delta, end));
}

BOOL CEServ::Load ()
{
	//   
	m_osv.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
	if (!GetVersionEx (&m_osv)) {
		ATLTRACE(_T("CEServ::Load () : GetVersionEx failed\n"));
		return FALSE;
	}

	// Under NT (and 95?), file open dialog crashes apps upon shutdown,
	// so we disable our icon overlays unless we are in Explorer process
	// space.
	BOOL allowExplorer = m_osv.dwMajorVersion > 4 || // allow anything major > 4
		(m_osv.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS &&
		m_osv.dwMajorVersion == 4 && m_osv.dwMinorVersion > 0); // plus Windows 98/Me
	if (!allowExplorer) {
		// ,     
		TCHAR buf [MAX_PATH];
		GetModuleFileName (NULL, buf, MAX_PATH);
		if (!is_at_end (buf, _T("explorer.exe"))) {
			ATLTRACE(_T("CEServ::Load () : Not in Explorer process : \"%s\"\n"), buf);
			return FALSE;
		}
	}

	//    EServ
	BOOL ret = FALSE;
	TCHAR tmp [MAX_PATH + 1];
	LPTSTR name;
	
	//    
	DWORD type = REG_SZ;
	DWORD size = MAX_PATH + 1;
	if (SHGetValue (HKEY_LOCAL_MACHINE, szUninstall, _T("UninstallString"),
		&type, tmp, &size) == ERROR_SUCCESS && type == REG_SZ) {
		name = StrRChr (tmp, NULL, _T('\\'));
		if (name)
			*name = _T('\0');
		if (SearchPath (tmp, szEServEXE, NULL, MAX_PATH + 1, tmp, &name)) {
			*name = _T('\0');
			ret = TRUE;
		}
	}

	//    
	if (!ret) {
		if (SearchPath (NULL, szEServEXE, NULL, MAX_PATH, tmp, &name)) {
			*name = _T('\0');
			ret = TRUE;
		}
	}
	if (ret) {
		PathAddBackslash (tmp);
		m_EServPath = tmp;
	} else {
		// -
		m_EServPath = _T("C:\\Program Files\\Eserv2\\");
	}

	ATLTRACE(_T("CEServ::Load () : EServ folder : \"%s\"\n"), m_EServPath);

	//    
	GetLongPathName (m_Path, tmp, MAX_PATH + 1);
	PathAddBackslash (tmp);
	m_Path = tmp;

	ATLTRACE(_T("CEServ::Load () : Target folder : \"%s\"\n"), m_Path);

	//  
	if (!LoadAliases ())
		return FALSE;
	
	// ,   
	m_State = m_InitialState = !m_Aliases.IsEmpty ();

	return TRUE;
}

void CEServ::Modify (BOOL modify)
{
	ATLTRACE(_T("CEServ::Modify\n"));

	if (modify)
		PropSheet_Changed (GetParent ().m_hWnd, m_hWnd);
	else
		PropSheet_UnChanged (GetParent ().m_hWnd, m_hWnd);
}

BOOL CEServ::StopEServ (EServState& state)
{
	state = ESERV_STOP;

	BOOL run;
	if (m_osv.dwPlatformId == VER_PLATFORM_WIN32_NT) {
		if (StopEServService (run) && run) {
			state = ESERV_SERVICE;
			return TRUE;
		}
	}
	if (StopEServStandalone (run)) {
		if (run) {
			state = ESERV_STANDALONE;
			return TRUE;
		}
	} else
		return FALSE;

	return TRUE;
}

BOOL CEServ::StopEServService (BOOL& state)
{
	ATLTRACE(_T("CEServ::StopEServService\n"));

	BOOL ret = FALSE;
	state = FALSE;
	SC_HANDLE hMan = OpenSCManager (NULL, NULL, SC_MANAGER_CONNECT);
	if (hMan) {
		SC_HANDLE hServ = OpenService (hMan, szEServService, SERVICE_STOP);
		if (hServ) {
			SERVICE_STATUS ss;
			if (ControlService (hServ, SERVICE_CONTROL_STOP, &ss)) {
				//    
				ATLTRACE(_T("Service found and stopped\n"));
				ret = TRUE;
				state = TRUE;
			} else {
				DWORD err = GetLastError ();
				if (err == ERROR_SERVICE_NOT_ACTIVE) {
					//    
					ATLTRACE(_T("Service found but not active\n"), err);
					ret = TRUE;
					state = FALSE;
				} else
					ATLTRACE(_T("ERROR : ControlService : 0x%08x\n"), err);
			}
			CloseServiceHandle (hServ);
		} else 
			ATLTRACE(_T("ERROR : OpenService : 0x%08x\n"), GetLastError ());
		CloseServiceHandle (hMan);
	} else 
		ATLTRACE(_T("ERROR : OpenSCManager : 0x%08x\n"), GetLastError ());

	return ret;
}

BOOL CALLBACK CloseAppEnum (HWND hwnd, LPARAM lParam) throw ()
{
	DWORD dwID;
	GetWindowThreadProcessId (hwnd, &dwID);
	if (dwID == (DWORD) lParam && GetParent (hwnd) == NULL) {
		if (PostMessage (hwnd, WM_CLOSE, 0, 0)) {
			ATLTRACE(_T("Close 0x%08x:0x%08x : OK\n"), dwID, hwnd);
		} else {
			ATLTRACE(_T("Close 0x%08x:0x%08x : FAILED\n"), dwID, hwnd);
		}
	}
	return TRUE;
}

BOOL CALLBACK PressAppEnum (HWND hwnd, LPARAM lParam) throw ()
{
	DWORD dwID;
	GetWindowThreadProcessId (hwnd, &dwID);
	if (dwID == (DWORD) lParam) {
		HWND child = GetWindow (hwnd, GW_CHILD);
		if (child) {
			DWORD id = GetWindowLong (child, GWL_ID);
			if (id == IDYES || id == IDOK || id == IDCANCEL) {
				if (PostMessage (hwnd, WM_COMMAND,
					MAKELONG (id, BN_CLICKED), (LPARAM) child)) {
					ATLTRACE(_T("Press 0x%08x:0x%08x : OK\n"), dwID, child);
				} else {
					ATLTRACE(_T("Press 0x%08x:0x%08x : FAILED\n"), dwID, child);
				}
			}
		}
	}
	return TRUE;
}

BOOL CEServ::StopEServStandalone (BOOL& state)
{
	ATLTRACE(_T("CEServ::StopEServStandalone\n"));

	//     
	BOOL ret = FALSE;
	state = FALSE;
	HANDLE snapshot = CreateToolhelp32Snapshot (TH32CS_SNAPPROCESS, 0);
	if (snapshot != INVALID_HANDLE_VALUE) {
		PROCESSENTRY32 pe32;
		ZeroMemory (&pe32, sizeof (pe32));
		pe32.dwSize = sizeof (PROCESSENTRY32);
		if (Process32First (snapshot, &pe32)) {
			ret = TRUE;
			do {
				if (is_at_end (pe32.szExeFile, szEServEXE)) {
					// 
					state = TRUE;
					HANDLE process = OpenProcess (SYNCHRONIZE | PROCESS_TERMINATE,
						FALSE, pe32.th32ProcessID);
					if (process != INVALID_HANDLE_VALUE) {

						//    
						EnumWindows ((WNDENUMPROC) CloseAppEnum, pe32.th32ProcessID);

						//   
						int count = 10;
						while (WaitForSingleObject (process, 500) == WAIT_TIMEOUT) {
							if (count--) {
								//    
								ATLTRACE(_T("Try : %d\n"), count);
								EnumWindows ((WNDENUMPROC) PressAppEnum, pe32.th32ProcessID);
							} else {
								//  
								ATLTRACE(_T("Termination : FORCE\n"));
								TerminateProcess (process, 0);
							}
						}

						CloseHandle (process);
					} else {
						ret = FALSE;
						ATLTRACE(_T("ERROR : OpenProcess : 0x%08x\n"), GetLastError ());
					}
					break;
				}
			} while (Process32Next (snapshot, &pe32));
		} else
			ATLTRACE(_T("ERROR : Process32First : 0x%08x\n"), GetLastError ());
		CloseHandle (snapshot);
	} else 
		ATLTRACE(_T("ERROR : CreateToolhelp32Snapshot : 0x%08x\n"), GetLastError ());

	return ret;
}

BOOL CEServ::StartEServ (EServState state)
{
	switch (state) {
		case ESERV_STANDALONE:
			return StartEServStandalone ();
		case ESERV_SERVICE:
			return StartEServService ();
	}
	return TRUE;
}

BOOL CEServ::StartEServService ()
{
	ATLTRACE(_T("CEServ::StartEServService\n"));

	//  
	BOOL ret = FALSE;
	SC_HANDLE hMan = OpenSCManager (NULL, NULL, SC_MANAGER_CONNECT);
	if (hMan) {
		SC_HANDLE hServ = OpenService (hMan, szEServService, SERVICE_START);
		if (hServ) {
			if (StartService (hServ, 0, NULL)) {
				//    
				ATLTRACE(_T("Service started\n"));
				ret = TRUE;
			} else
				ATLTRACE(_T("ERROR : StartService : 0x%08x\n"), GetLastError ());
			CloseServiceHandle (hServ);
		} else 
			ATLTRACE(_T("ERROR : OpenService : 0x%08x\n"), GetLastError ());
		CloseServiceHandle (hMan);
	} else 
		ATLTRACE(_T("ERROR : OpenSCManager : 0x%08x\n"), GetLastError ());
	return ret;
}

BOOL CEServ::StartEServStandalone ()
{
	ATLTRACE(_T("CEServ::StartEServStandalone\n"));

	//  
	HINSTANCE inst = ShellExecute (NULL, NULL, m_EServPath + szEServEXE,
		_T("/ns"), m_EServPath, SW_SHOWMINIMIZED);
	if (inst > (HINSTANCE) 32) {
		// 
		ATLTRACE(_T("Standalone started \"%s\" : 0x%08x\n"),
			m_EServPath + szEServEXE, inst);
		return TRUE;
	} else 
		ATLTRACE(_T("ERROR : ShellExecute : 0x%08x\n"), GetLastError ());
	return FALSE;
}

void CEServ::SimplifyPath (CString& dst, const CString& src)
{
	//    
	TCHAR tmp1 [MAX_PATH + 1];
	GetShortPathName (m_EServPath, tmp1, MAX_PATH + 1);
	TCHAR tmp2 [MAX_PATH + 1];
	GetShortPathName (src, tmp2, MAX_PATH + 1);

	//   
	int len1 = lstrlen (tmp1);
	int len2 = lstrlen (tmp2);
	if (len1 > len2)
		len1 = 0;
	else
		if (CompareString (LOCALE_USER_DEFAULT, NORM_IGNORECASE,
			tmp1, len1, tmp2, len1) != CSTR_EQUAL)
			len1 = 0;
	dst = tmp2 + len1;
}

void CEServ::CanonicalizePath (CString& path)
{
	//  
	TCHAR* tmp1 = path.GetBuffer (MAX_PATH + 1);
	TCHAR tmp2 [MAX_PATH + 1];

	PathRemoveBlanks (/* inplace */ tmp1);
	convert (/* inplace */ tmp1, _T('/'), _T('\\'));
	PathCanonicalize (/* dst */ tmp2, /* src */ tmp1);
	PathCombine (/* dst */ tmp1, /* src */ m_EServPath, /* src */ tmp2);
	GetLongPathName (/* src */ tmp1, /* dst */ tmp2, MAX_PATH + 1);
	PathAddBackslash (/* inplace */ tmp2);
	lstrcpy (/* dst */ tmp1, /* src */ tmp2);

	path.ReleaseBuffer ();
}

BOOL CEServ::LoadACL (HKEY hRightsKey)
{
	ATLTRACE(_T("CEServ::LoadACL\n"));

	//   
	BOOL ret = TRUE;
	HKEY hKey;
	if (RegOpenKeyEx (hRightsKey, _T("Rights"), 0, KEY_READ, &hKey) == ERROR_SUCCESS) {
		TCHAR buf [30];
		DWORD dat;
		for (DWORD index = 0; ; index++) {
			DWORD buf_size = MAX_PATH;
			DWORD dat_size = sizeof (dat);
			DWORD type = REG_DWORD;
			LONG res = RegEnumValue (hKey, index, buf, &buf_size, 0,
				&type, reinterpret_cast <LPBYTE> (&dat), &dat_size);
			if ((res == ERROR_SUCCESS || res == ERROR_MORE_DATA) && type == REG_DWORD) {
				m_ACL.AddTail (EAC (buf, dat));
   				ATLTRACE(_T(" Found \"%s\" = 0x%08x\n"), buf, dat);
			} else
				break;
		}
		RegCloseKey (hKey);
	} else {
		ret = FALSE;
		ATLTRACE(_T("ERROR: 0x%08x\n"), GetLastError ());
	}
	return ret;
}

BOOL CEServ::AccessRights (BOOL load)
{
	ATLTRACE(_T("CEServ::AccessRights\n"));
	
	if (load)
		m_ACL.RemoveAll ();

	//      
	BOOL ret = TRUE;
	HKEY hKey;
	if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, szObjects,
		0, KEY_QUERY_VALUE | KEY_ENUMERATE_SUB_KEYS | (!load ? KEY_SET_VALUE : 0),
		&hKey) == ERROR_SUCCESS) {

		TCHAR buf [5];
		FILETIME time;
		for (DWORD index = 0; ; index++) {
			DWORD buf_size = 5;
			LONG res = RegEnumKeyEx (hKey, index, buf, &buf_size, 0, NULL, NULL, &time);
			if (res == ERROR_SUCCESS || res == ERROR_MORE_DATA) {
				
				//   
				BOOL found = FALSE;
				HKEY hObjectKey = NULL;
				if (RegOpenKeyEx (hKey, buf, 0, KEY_READ, &hObjectKey) == ERROR_SUCCESS) {
					DWORD type = REG_SZ;
					CString path;
					DWORD path_size = (MAX_PATH + 1) * sizeof (TCHAR);
					LONG err = RegQueryValueEx (hObjectKey, _T("Object"), NULL, &type,
						(LPBYTE) path.GetBuffer (MAX_PATH + 1), &path_size);
					path.ReleaseBuffer ();
					if (err == ERROR_SUCCESS && type == REG_SZ) {
						CanonicalizePath (path);
						found = !m_Path.CompareNoCase (path);
					} else {
						ret = FALSE;
						ATLTRACE(_T("ERROR: RegQueryValueEx : 0x%08x\n"), GetLastError ());
					}
				} else {
					ret = FALSE;
					ATLTRACE(_T("ERROR: RegOpenKeyEx (2) : 0x%08x\n"), GetLastError ());
				}
				if (found) {
					if (load) {
						//   
						LoadACL (hObjectKey);
						ATLTRACE(_T(" Found \"%s\"\n"), buf);
					} else {
						//   
						SHDeleteKey (hKey, buf);
						index = 0;
						ATLTRACE(_T(" Delete \"%s\"\n"), buf);
					}
				} else 
					ATLTRACE(_T(" Bypass \"%s\"\n"), buf);
				if (hObjectKey)
					RegCloseKey (hObjectKey);
			} else
				break;
		}
		RegCloseKey (hKey);
	} else {
		ret = FALSE;
		ATLTRACE(_T("ERROR: RegOpenKeyEx (1) : 0x%08x\n"), GetLastError ());
	}
	return ret;
}

void CEServ::LoadUsers ()
{
	ATLTRACE(_T("CEServ::LoadUsers\n"));

	m_Users.RemoveAll ();

	/*/    
	HKEY hKey;
	if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, szUsers, 0, KEY_READ, &hKey) == ERROR_SUCCESS) {

		TCHAR buf [30];
		FILETIME time;
		for (DWORD index = 0; ; index++) {
			DWORD buf_size = 30;
			LONG res = RegEnumKeyEx (hKey, index, buf, &buf_size, 0, NULL, NULL, &time);
			if (res == ERROR_SUCCESS || res == ERROR_MORE_DATA) {
				
				ATLTRACE(_T(" -> %s\n"), buf);

				m_Users.AddTail (buf);	
			} else
				break;
		}

		RegCloseKey (hKey);
	} else
		ATLTRACE(_T("ERROR: 0x%08x\n"), GetLastError ());*/
}

void CEServ::LoadGroups ()
{
	ATLTRACE(_T("CEServ::LoadGroups\n"));

	m_Groups.RemoveAll ();

	//  
	m_Groups.AddTail (_T("All"));
	m_Groups.AddTail (_T("AllUsers"));

	//    
	HKEY hKey;
	if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, szGroups, 0, KEY_READ, &hKey) == ERROR_SUCCESS) {

		TCHAR buf [30];
		FILETIME time;
		for (DWORD index = 0; ; index++) {
			DWORD buf_size = 30;
			LONG res = RegEnumKeyEx (hKey, index, buf, &buf_size, 0, NULL, NULL, &time);
			if (res == ERROR_SUCCESS || res == ERROR_MORE_DATA) {
				
				ATLTRACE(_T(" -> %s\n"), buf);

				m_Groups.AddTail (buf);	
			} else
				break;
		}

		RegCloseKey (hKey);
	} else
		ATLTRACE(_T("ERROR: 0x%08x\n"), GetLastError ());
}

BOOL CEServ::Aliases (BOOL load)
{
	ATLTRACE(_T("CEServ::Aliases\n"));

	if (load)
		m_Aliases.RemoveAll ();

	BOOL ret = TRUE;
	HKEY hKey;
	if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, szVirtualFolders,
		0, KEY_QUERY_VALUE | KEY_ENUMERATE_SUB_KEYS | (!load ? KEY_SET_VALUE : 0),
		&hKey) == ERROR_SUCCESS) {

		//   
		TCHAR buf [MAX_PATH + 1];
		CString dat;
		for (DWORD index = 0; ; index++) {
			DWORD buf_size = (MAX_PATH + 1) * sizeof (TCHAR);
			DWORD dat_size = (MAX_PATH + 1) * sizeof (TCHAR);
			LONG err = RegEnumValue (hKey, index, buf, &buf_size, 0, NULL,
				(LPBYTE) dat.GetBuffer (MAX_PATH + 1), &dat_size);
			dat.ReleaseBuffer ();
			if (err == ERROR_SUCCESS || err == ERROR_MORE_DATA) {

				//  
				CanonicalizePath (dat);		
				if (!m_Path.CompareNoCase (dat)) {
					if (load) {

						//  
						m_Aliases.AddTail (buf);
						ATLTRACE (_T(" Found \"%s\" = \"%s\"\n"), buf, dat);
					} else {

						//  
						if (RegDeleteValue (hKey, buf) == ERROR_SUCCESS) {
							index = 0;
							ATLTRACE (_T(" Delete \"%s\" = \"%s\"\n"), buf, dat);
						} else {
							ATLTRACE(_T("ERROR: 0x%08x\n"), GetLastError ());
							ret = FALSE;
						}
					}
				} else 
					ATLTRACE(_T(" Bypass \"%s\" = \"%s\"\n"), buf, dat);
			} else
				break;
		}
		RegCloseKey (hKey);
	} else {
		ATLTRACE(_T("ERROR: 0x%08x\n"), GetLastError ());
		ret = FALSE;
	}
	return ret;
}

BOOL CEServ::SaveAccessRights ()
{
	ATLTRACE (_T("CEServ::SaveAccessRights\n"));

	//  
	ClearAccessRights ();

	if (!m_State)
		return TRUE;

	//    
	HKEY hKey;
	TCHAR key_name [MAX_PATH];
	lstrcpy (key_name, szObjects);
	LPTSTR sub_name = key_name + lstrlen (key_name);
	for (int i = 1; i < 1000; i++) {
		wnsprintf (sub_name, MAX_PATH, _T("\\%d"), i);
		if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, key_name, 0,
			KEY_QUERY_VALUE | KEY_ENUMERATE_SUB_KEYS, &hKey) != ERROR_SUCCESS) {
			break;
		} else
			RegCloseKey (hKey);
	}

	if (i == 1000)
		//  1000 ?!
		return FALSE;

	//   
	DWORD num = 0;
	ATLTRACE (_T(" Create \"%s\"\n"), key_name);
	if (RegCreateKeyEx (HKEY_LOCAL_MACHINE, key_name, 0, NULL,
		REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey, NULL) == ERROR_SUCCESS) {

		//  
		CString path;
		SimplifyPath (path, m_Path);
		if (RegSetValueEx (hKey, _T("Object"), 0, REG_SZ, (CONST LPBYTE) (LPCTSTR) path,
			(path.GetLength () + 1) * sizeof (TCHAR)) == ERROR_SUCCESS) {

			//   
			HKEY hRightsKey;
			if (RegCreateKeyEx (hKey, _T("Rights"), 0, NULL, REG_OPTION_NON_VOLATILE,
				KEY_WRITE, NULL, &hRightsKey, NULL) == ERROR_SUCCESS) {

				//  
				POSITION pos = m_ACL.GetHeadPosition ();
				while (pos) {

					EAC ac = m_ACL.GetNext (pos);					
					if (RegSetValueEx (hRightsKey, ac.subject, 0, REG_DWORD,
						(CONST LPBYTE) &ac.rights, sizeof (DWORD)) == ERROR_SUCCESS) {
						num++;
						ATLTRACE (_T(" Add \"%s\" = 0x%08x\n"), ac.subject, ac.rights);
					} else
						ATLTRACE(_T("ERROR: 0x%08x\n"), GetLastError ());
				}
				RegCloseKey (hRightsKey);
			} else 
				ATLTRACE(_T("ERROR: 0x%08x\n"), GetLastError ());
		} else 
			ATLTRACE(_T("ERROR: 0x%08x\n"), GetLastError ());
		RegCloseKey (hKey);
	} else 
		ATLTRACE(_T("ERROR: 0x%08x\n"), GetLastError ());

	return (num == m_ACL.GetCount ());
}

BOOL CEServ::SaveAliases ()
{
	ATLTRACE (_T("CEServ::SaveAliases\n"));

	//  
	ClearAliases ();

	if (!m_State)
		return TRUE;

	//  
	CString path;
	SimplifyPath (path, m_Path);
	int size = (path.GetLength () + 1) * sizeof (TCHAR);
	POSITION pos = m_Aliases.GetHeadPosition ();
	while (pos) {
		CString alias = m_Aliases.GetNext (pos);
		SHSetValue (HKEY_LOCAL_MACHINE, szVirtualFolders, alias, REG_SZ, path, size);
		ATLTRACE (_T(" Add \"%s\" = \"%s\"\n"), alias, m_Path);
	}

	return TRUE;
}

void CEServ::MirrorState (BOOL select)
{
	BOOL aliases_not_empty = !m_Aliases.IsEmpty ();
	BOOL acl_not_empty = !m_ACL.IsEmpty ();

	BOOL enabled;
	if (select) {
		enabled = m_InitialState;
		CheckDlgButton (IDC_ENABLE, (enabled ? BST_CHECKED : BST_UNCHECKED));
		CheckDlgButton (IDC_DISABLE, (enabled ? BST_UNCHECKED : BST_CHECKED));
	} else
		enabled = m_State;

	BOOL aliases_selected = aliases_not_empty &&
		(SendDlgItemMessage (IDC_ALIASES, LB_GETCURSEL, 0, 0) != LB_ERR);
	BOOL acl_selected = acl_not_empty &&
		(SendDlgItemMessage (IDC_ACL, LB_GETCURSEL, 0, 0) != LB_ERR);

	GetDlgItem (IDC_ADD_ALIAS).EnableWindow (enabled);
	GetDlgItem (IDC_EDIT_ALIAS).EnableWindow (enabled && aliases_selected);
	GetDlgItem (IDC_DEL_ALIAS).EnableWindow (enabled && aliases_selected);
	GetDlgItem (IDC_ALIASES).EnableWindow (enabled && aliases_not_empty);
	GetDlgItem (IDC_ADD_ACL).EnableWindow (enabled);
	GetDlgItem (IDC_EDIT_ACL).EnableWindow (enabled && acl_selected);
	GetDlgItem (IDC_DEL_ACL).EnableWindow (enabled && acl_selected);
	GetDlgItem (IDC_ACL).EnableWindow (enabled && acl_not_empty);
}

UINT CALLBACK CEServ::PageProc (HWND /*hwnd*/, UINT uMsg, LPPROPSHEETPAGE ppsp)
{
	CEServ* self = reinterpret_cast <CEServ*> (ppsp->lParam);
	switch (uMsg) {
		case PSPCB_ADDREF:
			ATLTRACE(_T("CEServ::PageProc -> PSPCB_ADDREF\n"));

			self->AddRef ();
			break;

		case PSPCB_CREATE:
			ATLTRACE(_T("CEServ::PageProc -> PSPCB_CREATE\n"));
			break;

		case PSPCB_RELEASE:
			ATLTRACE(_T("CEServ::PageProc -> PSPCB_RELEASE\n"));

			self->Release ();
			break;
	}
	return 1U;
}

// IShellExtInit

STDMETHODIMP CEServ::Initialize (
	LPCITEMIDLIST pidlFolder,
	LPDATAOBJECT pdobj,
	HKEY /*hkeyProgID*/)
{
	ATLTRACE(_T("CEServ::Initialize\n"));

	//  
	HRESULT res = E_INVALIDARG;
	if (!pidlFolder && pdobj) {
		FORMATETC fmte = { CF_HDROP, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
		STGMEDIUM medium = { TYMED_HGLOBAL };
		if (SUCCEEDED (pdobj->GetData (&fmte, &medium))) {
			HDROP hdrop = (HDROP) GlobalLock (medium.hGlobal);
			if (hdrop) {
				//   
				if (DragQueryFile (hdrop, (UINT) (-1), NULL, 0) == 1) {
					//  
					TCHAR path [MAX_PATH + 1];
					if (DragQueryFile (hdrop, 0, path, MAX_PATH + 1) > 0) {
						//  
						if (PathIsDirectory (path)) {
							m_Path = path;
							res = (Load () ? S_OK : E_FAIL);
						} else
							ATLTRACE(_T("ERROR: Not directory\n"));
					} else
						ATLTRACE(_T("ERROR: Multiple selection (2)\n"));
				} else
					ATLTRACE(_T("ERROR: Multiple selection (1)\n"));
				GlobalUnlock (medium.hGlobal);
			} else
				ATLTRACE(_T("ERROR: Cant lock data\n"));
			ReleaseStgMedium (&medium);
		} else
			ATLTRACE(_T("ERROR: Cant get data\n"));
	} else
		ATLTRACE(_T("ERROR: Invalid call\n"));
	return res;
}

// IShellPropSheetExt

STDMETHODIMP CEServ::AddPages (
	LPFNADDPROPSHEETPAGE lpfnAddPage,
	LPARAM lParam)
{
	ATLTRACE(_T("CEServ::AddPages\n"));

	PROPSHEETPAGE psp;
	ZeroMemory (&psp, sizeof (psp));
	psp.dwSize = sizeof (PROPSHEETPAGE);
	psp.dwFlags = PSP_DEFAULT | PSP_USECALLBACK;
	psp.hInstance = _AtlBaseModule.GetResourceInstance ();
	psp.pszTemplate = MAKEINTRESOURCE (IDD_ESERV);
	psp.pfnCallback = PageProc;
	psp.pfnDlgProc = StartDialogProc;
	psp.lParam = reinterpret_cast <LPARAM> (this);

	Create (NULL);

	HPROPSHEETPAGE hpage = CreatePropertySheetPage (&psp);
	if (hpage)
		if (!lpfnAddPage (hpage, lParam))
			DestroyPropertySheetPage (hpage);

	return S_OK;
}

void CEServ::FillAliases ()
{
	//   
	SendDlgItemMessage (IDC_ALIASES, LB_RESETCONTENT, 0, 0);
	POSITION pos = m_Aliases.GetHeadPosition ();
	while (pos) {
		POSITION current_pos = pos;

		LRESULT index = SendDlgItemMessage (IDC_ALIASES, LB_ADDSTRING, 0,
			reinterpret_cast <LPARAM> ((LPCTSTR) m_Aliases.GetNext (pos)));
		SendDlgItemMessage (IDC_ALIASES, LB_SETITEMDATA, index,
			reinterpret_cast <LPARAM> (current_pos));
	}
}

void CEServ::FillACL ()
{
	//   
	SendDlgItemMessage (IDC_ACL, LB_RESETCONTENT, 0, 0);
	POSITION pos = m_ACL.GetHeadPosition ();
	while (pos) {
		POSITION current_pos = pos;
		EAC ac (m_ACL.GetNext (pos));

		// DLRWX
		CString tmp;
		tmp.Format (_T("%s   [%s%s%s%s%s]"), (LPCTSTR) ac.subject,
			((ac.rights & 16) ? "D" : ""),
			((ac.rights & 8) ? "L" : ""),
			((ac.rights & 4) ? "R" : ""),
			((ac.rights & 2) ? "W" : ""),
			((ac.rights & 1) ? "X" : ""));

		LRESULT index = SendDlgItemMessage (IDC_ACL, LB_ADDSTRING, 0,
			reinterpret_cast <LPARAM> ((LPCTSTR) tmp));
		SendDlgItemMessage (IDC_ACL, LB_SETITEMDATA, index,
			reinterpret_cast <LPARAM> (current_pos));
	}
}

LRESULT CEServ::OnInitDialog (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	CAxDialogImpl<CEServ>::OnInitDialog (uMsg, wParam, lParam, bHandled);

//	PROPSHEETPAGE* ppsp = reinterpret_cast <PROPSHEETPAGE*> (lParam);

	ATLTRACE(_T("CEServ::OnInitDialog [InitialState = %d]\n"), m_InitialState);

	LoadUsers ();
	LoadGroups ();

	if (!LoadAccessRights ())
		return FALSE;

	FillAliases ();
	FillACL ();
	MirrorState (TRUE);

	return 1;  // Let the system set the focus
}

LRESULT CEServ::OnClickedEnable (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_State = (IsDlgButtonChecked (IDC_ENABLE) == BST_CHECKED);

	if (m_State != m_InitialState)
		Modify ();

	MirrorState ();
	return 0;
}

LRESULT CEServ::OnClickedDisable (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_State = (IsDlgButtonChecked (IDC_ENABLE) == BST_CHECKED);

	if (m_State != m_InitialState)
		Modify ();

	MirrorState ();
	return 0;
}

LRESULT CEServ::OnClickedAddAlias (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	//  
	CAddAlias dlg;
	dlg.m_Alias = _T("");
	if (dlg.DoModal () == IDOK) {
		//  
		dlg.m_Alias.Replace (_T('\\'), _T('/'));
		if (dlg.m_Alias [0] != _T('/'))
			dlg.m_Alias.Insert (0, _T('/'));
		if (dlg.m_Alias [dlg.m_Alias.GetLength () - 1] != _T('/'))
			dlg.m_Alias.AppendChar (_T('/'));

		m_Aliases.AddTail (dlg.m_Alias);
		FillAliases ();

		Modify ();
	}
	MirrorState ();
	return 0;
}

LRESULT CEServ::OnBnClickedEditAlias(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	//  
	LRESULT index = SendDlgItemMessage (IDC_ALIASES, LB_GETCURSEL, 0, 0);
	if (index != LB_ERR) {
		POSITION pos = reinterpret_cast <POSITION> (
			SendDlgItemMessage (IDC_ALIASES, LB_GETITEMDATA, index, 0));

		CAddAlias dlg;
		dlg.m_Alias = m_Aliases.GetAt (pos);
		if (dlg.DoModal () == IDOK) {
			//  
			dlg.m_Alias.Replace (_T('\\'), _T('/'));
			if (dlg.m_Alias [0] != _T('/'))
				dlg.m_Alias.Insert (0, _T('/'));
			if (dlg.m_Alias [dlg.m_Alias.GetLength () - 1] != _T('/'))
				dlg.m_Alias.AppendChar (_T('/'));

			//    
			m_Aliases.RemoveAt (pos);
			m_Aliases.AddTail (dlg.m_Alias);
			FillAliases ();

			Modify ();
		}
	}

	return 0;
}

LRESULT CEServ::OnClickedDelAlias (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	//  
	LRESULT index = SendDlgItemMessage (IDC_ALIASES, LB_GETCURSEL, 0, 0);
	if (index != LB_ERR) {
		POSITION pos = reinterpret_cast <POSITION> (
			SendDlgItemMessage (IDC_ALIASES, LB_GETITEMDATA, index, 0));
		m_Aliases.RemoveAt (pos);
		SendDlgItemMessage (IDC_ALIASES, LB_DELETESTRING, index, 0);

		if (index)
			SendDlgItemMessage (IDC_ALIASES, LB_SETCURSEL, index - 1, 0);
		else
			if (!m_Aliases.IsEmpty ())
				SendDlgItemMessage (IDC_ALIASES, LB_SETCURSEL, 0, 0);

		Modify ();
	}
	MirrorState ();
	return 0;
}

LRESULT CEServ::OnSelectedAlias (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	MirrorState ();
	return 0;
}

LRESULT CEServ::OnBnClickedAddAcl(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	//  
	CAccessRights dlg;
	dlg.m_UserEnabled = TRUE;
	dlg.m_pGroups = &m_Groups;
	dlg.m_pUsers = &m_Users;
	if (dlg.DoModal () == IDOK) {
		m_ACL.AddTail (dlg.m_AC);
		FillACL ();

		Modify ();
	}

	MirrorState ();
	return 0;
}

LRESULT CEServ::OnBnClickedEditAcl(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	//  
	LRESULT index = SendDlgItemMessage (IDC_ACL, LB_GETCURSEL, 0, 0);
	if (index != LB_ERR) {
		POSITION pos = reinterpret_cast <POSITION> (
			SendDlgItemMessage (IDC_ACL, LB_GETITEMDATA, index, 0));

		CAccessRights dlg;
		dlg.m_UserEnabled = FALSE;
		dlg.m_AC = m_ACL.GetAt (pos);
		if (dlg.DoModal () == IDOK) {

			//    
			m_ACL.RemoveAt (pos);
			m_ACL.AddTail (dlg.m_AC);
			FillACL ();

			Modify ();
		}
	}
	MirrorState ();
	return 0;
}

LRESULT CEServ::OnBnClickedDelAcl(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	//  
	LRESULT index = SendDlgItemMessage (IDC_ACL, LB_GETCURSEL, 0, 0);
	if (index != LB_ERR) {
		POSITION pos = reinterpret_cast <POSITION> (
			SendDlgItemMessage (IDC_ACL, LB_GETITEMDATA, index, 0));
		m_ACL.RemoveAt (pos);
		SendDlgItemMessage (IDC_ACL, LB_DELETESTRING, index, 0);

		if (index)
			SendDlgItemMessage (IDC_ACL, LB_SETCURSEL, index - 1, 0);
		else
			if (!m_ACL.IsEmpty ())
				SendDlgItemMessage (IDC_ACL, LB_SETCURSEL, 0, 0);

		Modify ();
	}
	MirrorState ();
	return 0;
}

LRESULT CEServ::OnSelectedAcl (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	MirrorState ();
	return 0;
}

LRESULT CEServ::OnApply (int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
{
	ATLTRACE(_T("CEServ::OnApply\n"));

	//   
	if (m_State) {
		if (m_Aliases.IsEmpty ()) {
			MsgBox (_T("  ."));
			return PSNRET_INVALID;
		}
		if (m_ACL.IsEmpty ()) {
			MsgBox (_T("  ."));
			return PSNRET_INVALID;
		}
	}

	EServState state;
	if (!StopEServ (state)) {
		MsgBox (_T("  EServ"));
		return PSNRET_INVALID;
	}

	if (!SaveAccessRights ()) {
		MsgBox (_T("    "));
		return PSNRET_INVALID;
	}

	if (!SaveAliases ()) {
		MsgBox (_T("    "));
		return PSNRET_INVALID;
	}

	if (!StartEServ (state)) {
		MsgBox (_T("  EServ"));
		return PSNRET_INVALID;

	}

	SHChangeNotify (SHCNE_UPDATEDIR | (m_State ? SHCNE_NETSHARE : SHCNE_NETUNSHARE),
		SHCNF_PATH | SHCNF_FLUSHNOWAIT, m_Path, NULL);

	return PSNRET_NOERROR;
}

STDMETHODIMP CEServ::IsMemberOf (LPCWSTR pwszPath, DWORD /*dwAttrib*/)
{
	if (PathIsDirectoryW (pwszPath)) {	
		//  
		m_Path = pwszPath;
		if (Load () && m_State)
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CEServ::GetOverlayInfo (LPWSTR pwszIconFile, int cchMax,
									 int* pIndex, DWORD* pdwFlags)
{
	GetModuleFileNameW (_AtlBaseModule.GetModuleInstance(), pwszIconFile, cchMax);
	*pIndex = 0;
	*pdwFlags = ISIOI_ICONFILE | ISIOI_ICONINDEX;
	return S_OK;
}

STDMETHODIMP CEServ::GetPriority (int* pIPriority)
{
	//   (0..100)
	*pIPriority = 10;
	return S_OK;
}
