#pragma once

const LPCTSTR szEServService = _T("eserv.exe");
const LPCTSTR szObjects = _T("SOFTWARE\\Etype\\Eserv\\CommonSettings\\AccessRights\\Objects");
const LPCTSTR szVirtualFolders = _T("SOFTWARE\\Etype\\Eserv\\WebServer\\VirtualFolders");
const LPCTSTR szGroups = _T("SOFTWARE\\Etype\\Eserv\\CommonSettings\\UserGroups");
const LPCTSTR szUsers = _T("SOFTWARE\\Etype\\Eserv\\CommonSettings\\Users");
const LPCTSTR szUninstall = _T("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Eserv2");
const LPCTSTR szEServEXE = _T("Eserv.exe");

class CEServ :
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CEServ, &CLSID_EServ>,
	public IShellExtInit,
	public IShellPropSheetExt,
	public IShellIconOverlayIdentifier,
	public CAxDialogImpl<CEServ>
{
protected:
	enum EServState {ESERV_STOP, ESERV_SERVICE, ESERV_STANDALONE};

	CAtlStringList m_Aliases;		//  
	CAtlStringList m_Users;			//  
	CAtlStringList m_Groups;		//  
	EACL m_ACL;						//  
	CString m_Path;					//    
	CString m_EServPath;			//    EServ
	BOOL m_InitialState;			//   (/)
	BOOL m_State;					//   (/)
	OSVERSIONINFO m_osv;			//  

	//  
	void SimplifyPath (CString& dst, const CString& src);
	void CanonicalizePath (CString& path);

	//  EServ.exe
	BOOL StopEServ (EServState& state);
	BOOL StopEServService (BOOL& state);
	BOOL StopEServStandalone (BOOL& state);
	BOOL StartEServ (EServState state);
	BOOL StartEServService ();
	BOOL StartEServStandalone ();

	//  
	BOOL Load ();

	// / 
	inline BOOL LoadAccessRights () throw ()
	{
		return AccessRights (TRUE);
	}
	inline BOOL ClearAccessRights () throw ()
	{
		return AccessRights (FALSE);
	}
	BOOL SaveAccessRights ();
	BOOL AccessRights (BOOL load);
	BOOL LoadACL (HKEY hRightsKey);

	// / 
	inline BOOL LoadAliases () throw ()
	{
		return Aliases (TRUE);
	}
	inline BOOL ClearAliases () throw ()
	{
		return Aliases (FALSE);
	}
	BOOL SaveAliases ();
	BOOL Aliases (BOOL load);

	//  
	void LoadUsers ();

	//  
	void LoadGroups ();

	//  
	void FillAliases ();
	void FillACL ();
	void Modify (BOOL modify = TRUE);
	void MirrorState (BOOL select = FALSE);
	static UINT CALLBACK PageProc (HWND hwnd, UINT uMsg, LPPROPSHEETPAGE ppsp);
	void MsgBox (LPCTSTR szText) throw ()
	{ 
		MessageBox (szText, _T("EShell"), MB_OK | MB_ICONWARNING);
	}

public:
	CEServ () throw () : m_InitialState (FALSE), m_State (FALSE) {}

	enum { IDD = IDD_ESERV };

DECLARE_PROTECT_FINAL_CONSTRUCT()
DECLARE_REGISTRY_RESOURCEID(IDR_ESERV)

BEGIN_COM_MAP(CEServ) 
	COM_INTERFACE_ENTRY(IShellExtInit)
	COM_INTERFACE_ENTRY(IShellPropSheetExt)
	COM_INTERFACE_ENTRY(IShellIconOverlayIdentifier)
END_COM_MAP()

BEGIN_MSG_MAP(CEServ)
	MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
	COMMAND_HANDLER(IDC_ENABLE, BN_CLICKED, OnClickedEnable)
	COMMAND_HANDLER(IDC_DISABLE, BN_CLICKED, OnClickedDisable)
	COMMAND_HANDLER(IDC_ADD_ALIAS, BN_CLICKED, OnClickedAddAlias)
	COMMAND_HANDLER(IDC_EDIT_ALIAS, BN_CLICKED, OnBnClickedEditAlias)
	COMMAND_HANDLER(IDC_ALIASES, LBN_DBLCLK, OnBnClickedEditAlias)
	COMMAND_HANDLER(IDC_DEL_ALIAS, BN_CLICKED, OnClickedDelAlias)
	COMMAND_HANDLER(IDC_ALIASES, LBN_SELCHANGE, OnSelectedAlias)
	COMMAND_HANDLER(IDC_ADD_ACL, BN_CLICKED, OnBnClickedAddAcl)
	COMMAND_HANDLER(IDC_EDIT_ACL, BN_CLICKED, OnBnClickedEditAcl)
	COMMAND_HANDLER(IDC_ACL, LBN_DBLCLK, OnBnClickedEditAcl)
	COMMAND_HANDLER(IDC_DEL_ACL, BN_CLICKED, OnBnClickedDelAcl)
	COMMAND_HANDLER(IDC_ACL, LBN_SELCHANGE, OnSelectedAcl)
	NOTIFY_HANDLER(0, PSN_APPLY, OnApply)
	CHAIN_MSG_MAP(CAxDialogImpl<CEServ>)
END_MSG_MAP()

// IShellExtInit
public:
	STDMETHOD(Initialize)(
		LPCITEMIDLIST pidlFolder,
		LPDATAOBJECT lpdobj,
		HKEY hkeyProgID);

// IShellPropSheetExt
public:
	STDMETHOD(AddPages)(
		LPFNADDPROPSHEETPAGE lpfnAddPage,
		LPARAM lParam);

	STDMETHOD(ReplacePage)(
		UINT /*uPageID*/,
		LPFNADDPROPSHEETPAGE /*lpfnReplacePage*/,
		LPARAM /*lParam*/)
	{
		ATLTRACENOTIMPL(_T("CEServ::ReplacePage\n"));
	}

// IShellIconOverlayIdentifier
public:
	STDMETHOD (IsMemberOf)(
		LPCWSTR pwszPath,
		DWORD dwAttrib);
	STDMETHOD (GetOverlayInfo)(
		LPWSTR pwszIconFile,
		int cchMax,
		int * pIndex,
		DWORD * pdwFlags);
	STDMETHOD (GetPriority)(
		int * pIPriority);

// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

	LRESULT OnInitDialog (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnClickedEnable (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnClickedDisable (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnClickedAddAlias (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnBnClickedEditAlias(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnClickedDelAlias (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnSelectedAlias (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnBnClickedAddAcl(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnBnClickedEditAcl(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnBnClickedDelAcl(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnSelectedAcl (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnApply (int idCtrl, LPNMHDR pnmh, BOOL& bHandled);
};

OBJECT_ENTRY_AUTO(__uuidof(EServ), CEServ)
